@@ -1,8 +1,9 @@
 """aws_cdk_constructs package."""
 from .microservice import Microservice
 from .database import Database
-from .buckets import Buckets
+from .bucket import Bucket
+from .api import API
 from .service_user_for_iac import ServiceUserForIAC
 from .service_user_for_static_assets import ServiceUserForStaticAssets
 
-__version__ = '0.0.3'
\ No newline at end of file
+__version__ = '0.0.6'
\ No newline at end of file
