@@ -6,6 +6,7 @@ from aws_cdk import (
     aws_secretsmanager as _secretsmanager,
     aws_kms as _kms,
 )
+from aws_cdk.core import Tags
 
 engine_to_cdk_engine = {
     "legacy-aurora-mysql": _rds.DatabaseClusterEngine.aurora,
@@ -37,27 +38,76 @@ engine_to_version_class = {
     "postgresql": _rds.PostgresEngineVersion
 }
 
-
 class Database(core.Construct):
     """
-    A CDK construct to create a "computational tier" for your system.
-    The construct will make easy to develop a fully compliant macro infrastructure component that includes EC2 instances, served by an Application Load Balancer.
+    A CDK construct to create a "database tier" for your system.
+    The construct will make easy to develop a fully compliant database component that includes RDS cluster or instance.
+
+    Args:
+
+        id (str): the logical id of the newly created resource
+        
+        app_name (str): The application name. This will be used to generate the 'ApplicationName' tag for CSI compliancy. The ID of the application. This must be unique for each system, as it will be used to calculate the AWS costs of the system
+
+        environment (str): Specify the environment in which you want to deploy you system. Allowed values: Development, QA, Production, SharedServices 
+
+        environments_parameters (dict): The dictionary containing the references to CSI AWS environments. This will simplify the environment promotions and enable a parametric development of the infrastructures.
+
+        database_instance_type (str): 
+
+        database_name (str): Default=fao_default_schema The main database schema name (for 'Production' environment this must be 'fao_default_schema')
+        
+        database_master_username (str): Default="faoadmin" he database admin account username (for 'Production' environment this must be 'faoadmin')
+        
+        database_snapshot_id (str): The ARN of the Database Snapshot to restore. The snapshost contains the data that will be inserted into the database. Note that if specified, "DatabaseName" parameter will be ignored.
+        
+        database_engine (str): The engine of database you want to create
+
+        database_engine_version (str): The engine version of database you want to create. Leave blank to get the latest version of the selected database engine (MySQL 5.7, PostgreSQL 10). More info https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-engineversion
+        
+        database_cluster_parameters_group_name (str):  The name of the DB cluster parameter group to associate with this DB cluster. This parameter depends on the Database Engine version you previously selected. In case you leave blank the version use default.aurora-mysql5.7 or default.aurora-postgresql10. If this argument is omitted, default.aurora5.6 is used. If default.aurora5.6 is used, specifying aurora-mysql or aurora-postgresql for the Engine property might result in an error. More info https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-engineversion
+        
+        parameter_group (aws_rds.ParameterGroup): The parameter group to assign to the database
+
+        database_allocated_storage (str): The size of the allocated space for the database. This is GB
+        
+        database_will_send_email (str): If the database should send email
 
     """
     @property
     def security_group(self):
+        """Return the database security group
+        
+        Returns:
+            aws_ec2.SecurityGroup: the database security group
+        """ 
         return self._rds_security_group
 
     @property
     def cluster(self):
+        """Return the database cluster
+        
+        Returns:
+            aws_rds.DatabaseCluster: the database cluster
+        """ 
         return self._cluster
 
     @property
     def instance(self):
+        """Return the database instance
+        
+        Returns:
+            aws_rds.DatabaseInstance: the database instance
+        """  
         return self._instance
 
     @staticmethod
     def get_engine(database_engine, database_engine_version):
+        """Return the database engine as string
+        
+        Returns:
+            str: the database engine as string
+        """  
         # For MySQL engine: extract db major version
         major_version = database_engine_version.split(".")
         major_version = major_version[:-1]
@@ -74,9 +124,9 @@ class Database(core.Construct):
         self,
         scope: core.Construct,
         id:str,
-        environments_parameters=None,
-        environment=None,
-        app_name=None,
+        app_name,
+        environment,
+        environments_parameters,
         database_instance_type=None,
         database_name=None,
         database_master_username="faoadmin",
@@ -91,6 +141,10 @@ class Database(core.Construct):
     ):
         super().__init__(scope, id, **kwargs)
 
+        # Apply mandatory tags
+        Tags.of(self).add("ApplicationName", app_name.lower().strip())
+        Tags.of(self).add("Environment", environment)
+
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Create conditions
 
         environment = environment.lower()
