@@ -10,7 +10,9 @@ from aws_cdk import (
     aws_iam as _iam,
     aws_efs as _efs,
     aws_kms as _kms,
+    aws_ssm as _ssm
 )
+from aws_cdk.core import Tags
 import json
 import os
 import re
@@ -36,11 +38,13 @@ class Microservice(core.Construct):
 
 
     Args:
-        access_log_bucket_name (str): To enable Load Balancer acces logs to be stored in the specified S3 bucket
+        id (str): the logical id of the newly created resource
         
-        additional_variables (str): You can specify additional parameters that will be available as environment variables for the EC2 user-data script
+        access_log_bucket_name (str): Default: "fao-elb-logs". To enable Load Balancer acces logs to be stored in the specified S3 bucket
         
-        app_name (str): The application name. This will be used to generate the 'ApplicationName' tag for CSI compliancy. The ID of the application. This must be unique for each system, as it will be used to calculate the AWS costs of the systen
+        additional_variables (dict): You can specify additional parameters that will be available as environment variables for the EC2 user-data script
+        
+        app_name (str): The application name. This will be used to generate the 'ApplicationName' tag for CSI compliancy. The ID of the application. This must be unique for each system, as it will be used to calculate the AWS costs of the system
 
         authorization_endpoint (str): Used with 'authorization_endpoint', 'client_id', 'client_secret', 'issuer', 'token_endpoint', 'user_info_endpoint'. Used to perform the OIDC Cognito integration with the Application Load balancer. More information https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listenerrule-authenticateoidcconfig.html
 
@@ -57,7 +61,7 @@ class Microservice(core.Construct):
 
         ebs_volume_size (str): In case you want to create a secondary EBS volume for your EC2 instance, this parameter is used to specify the volume size. The parameter specify the desired GB. Only use this parameter when your system cannot horizontally scale!
 
-        ec2_ami_id (str): Specify the EC2 AMI id to use to create the EC2 instance.
+        ec2_ami_id (str): Specify the EC2 AMI id to use to create the EC2 instance. In no value is specified the laster Linux Hardened AMI will be used.
 
         ec2_health_check_path (str):  Specify the Target Group health check path to use to monitor the state of the EC2 instances. EC2 instances will be constantly monitored, performing requests to this path. The request must receive a successful response code to consider the EC2 healthy. Otherwise the EC2 will be terminated and regenerated. It must start with a slash "/"
 
@@ -79,8 +83,6 @@ class Microservice(core.Construct):
 
         ssl_certificate_arn (str): In case you want to enable HTTPS for your stack, specify the SSL certificate ARN to use. This configuration will force the creation of 2 load balancer Listeners (str)one on port 443 that proxies to the Target Group, a second one on port 80 to redirect the traffic to port 443 and enforce HTTPS. In case the application implements HTTPS, specify the ARN of the SSL Certificate to use. You can find it in AWS Certificate Manager
 
-        toggle_to_trigger_rolling_update (str): Utility paremeter to manually trigger a complete replacement of the EC2 instances fleet. This is used to implement continuous deployment.
-
         token_endpoint (str): Used with 'authorization_endpoint', 'client_id', 'client_secret', 'issuer', 'token_endpoint', 'user_info_endpoint'. Used to perform the OIDC Cognito integration with the Application Load balancer. More information https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listenerrule-authenticateoidcconfig.html
 
         traffic_port (str): Specify the port the Application Load Balancer will listen to. This is the port the final users will contact to browse the system. If HTTPS is enabled, this parameter will be forced to be 443. This is the port that the application will use to accpet traffic (e.g. if the application uses HTTPS, specify 443; if the application uses HTTP, specify 80; etc.)
@@ -103,29 +105,38 @@ class Microservice(core.Construct):
 
     @property
     def vpc(self):
-        """
+        """[summary]
         Returns the VPC in which the stack is deployed on
+
+        Returns:
+            aws_ec2.Vpc: the VPC in which the stack is deployed on
         """
         return self._vpc
 
     @property
     def alb_logs_bucket(self):
-        """
-        Returns S3 bucket that the Application Load Balancer is using for storing the logs
+        """Returns S3 bucket that the Application Load Balancer is using for storing the logs
+
+        Returns:
+            str: the S3 bucket that the Application Load Balancer is using for storing the logs
         """
         return self._alb_logs_bucket
 
     @property
     def tcp_connection_ec2_traffic_port(self):
-        """
-        Returns the EC2 traffic port as TCP connection
+        """Returns the EC2 traffic port as TCP connection
+
+        Returns:
+            aws_ec2.Port.tcp: the EC2 traffic port
         """
         return self._tcp_connection_ec2_traffic_port
 
     @property
     def tcp_connection_traffic_port(self):
-        """
-        Returns the Load Balancer port as TCP connection
+        """Returns the Load Balancer port as TCP connection
+
+        Returns:
+            aws_ec2.Port.tcp: the Load Balancer port
         """
         return self._tcp_connection_traffic_port
 
@@ -140,42 +151,57 @@ class Microservice(core.Construct):
 
     @property
     def auto_scaling_group(self):
-        """
-        Returns the Auto Scaling Group object
+        """Returns the Auto Scaling Group object
+
+        Returns:
+            aws_ec2.AutoScalingGroup: the Auto Scaling Group
         """
         return self._asg
 
     @property
     def load_balancer(self):
-        """
-        Returns the Application Load Balancer object
+        """Returns the Application Load Balancer object
+
+        Returns:
+            aws_alb.ApplicationLoadBalancer: the Load Balancer
         """
         return self.alb
 
     @property
     def load_balancer_security_group(self):
+        """Returns the security group in use by the application load balancer
+
+        Returns:
+            aws_ec2.SecurityGroup: the security group in use by the application load balancer
         """
-        Returns the security group in use by the application load balancer
-        """ 
         return self.alb_security_group
 
     @property
     def ec2_instance_security_group(self):
+        """Return the security group in use by the EC2 instance
+
+        Returns:
+            aws_ec2.SecurityGroup: the security group in use by the EC2 instance
         """
-        Return the security group in use by the EC2 instance
-        """ 
         return self.ec2_security_group
 
     @property
     def user_data(self):
+        """Return the user-data used by the EC2 instance on boot
+
+        Returns:
+            aws_ec2.UserData: the user-data used by the EC2 instance on boot
         """
-        Return the user-data used by the EC2 instance on boot
-        """ 
         return self.base_user_data
 
     def enable_fao_private_access(self, security_group):
-        """
-        Apply the correct ingress rules to the provided security group to enable access from the FAO internal networks
+        """Apply the correct ingress rules to the provided security group to enable access from the FAO internal networks
+        
+        Args:
+            security_group (aws_ec2.SecurityGroup): The security group to configure to enable access from FAO network
+    
+        Returns:
+            aws_ec2.SecurityGroup: the provided security group
         """ 
         security_group.add_ingress_rule(
             peer=self.pulse_cidr_as_peer(self._environments_parameters),
@@ -189,17 +215,34 @@ class Microservice(core.Construct):
             description="FAO HQ Clients",
         )
 
+        return security_group
+
     def create_target_group(
         self,
         id,
         app_name,
+        environment,
         ec2_traffic_port,
         ec2_health_check_path,
         protocol=_alb.ApplicationProtocol.HTTP,
     ):
-        """
-        Create a fully compliant Target Group object
+        """Create a Target Group object
+        
+        Args:
+            id (str): the logical id of the newly created resource
+
+            app_name (str): The application name. This will be used to generate the 'ApplicationName' tag for CSI compliancy. The ID of the application. This must be unique for each system, as it will be used to calculate the AWS costs of the system
+
+            ec2_traffic_port (str): Specify the port the EC2 instance will listen to. This is used also as the Target Group Health check configuration port. For example (str)if you EC2 is equipped with an Apache Tomcat, listening on port 8080, use this parameter to specify 8080. It's important to note that this is not port the final user will use to connect to the system, as the Load Balancer will be in-front of the EC2.This is the port that the load balancer will use to forward traffic to the EC2 (e.g. Tomacat uses port 8080, Node.js uses port 3000).
+
+            ec2_health_check_path (str):  Specify the Target Group health check path to use to monitor the state of the EC2 instances. EC2 instances will be constantly monitored, performing requests to this path. The request must receive a successful response code to consider the EC2 healthy. Otherwise the EC2 will be terminated and regenerated. It must start with a slash "/"
+            
+            protocol (aws_alb.ApplicationProtocol): Default=_alb.ApplicationProtocol.HTTP. The protocol the target group uses to receive traffic
+        
+        Returns:
+            aws_ec2.SecurityGroup: the provided security group
         """ 
+
         tg = _alb.ApplicationTargetGroup(
             self,
             id,
@@ -221,11 +264,26 @@ class Microservice(core.Construct):
             unhealthy_threshold_count=2,
         )
 
+        # Apply mandatory tags
+        Tags.of(tg).add("ApplicationName", app_name.lower().strip())
+        Tags.of(tg).add("Environment", environment)
+
         return tg
 
-    def create_load_balancer(self, scope, id, security_group, internet_facing=False, load_balancer_name=None):
-        """
-        Create a fully compliant Application Load Balancer object
+    def create_load_balancer(self, scope, id, app_name, environment, security_group, internet_facing=False, load_balancer_name=None):
+        """Create a Load Balancer resource
+        
+        Args:
+            id (str): the logical id of the newly created resource
+
+            security_group (aws_ec2.SecurityGroup): The security group to configure to enable access from FAO network
+
+            internet_facing (str): if the load balancer should be internet-facing or not
+
+            load_balancer_name (str): the load balancer name
+    
+        Returns:
+            aws_alb.ApplicationLoadBalancer: the newly created Load Balancer
         """ 
         if load_balancer_name is None:
             load_balancer_name = id
@@ -241,13 +299,27 @@ class Microservice(core.Construct):
         )
         # Enable logging
         alb.log_access_logs(self._alb_logs_bucket, prefix=self._app_name)
+
+        # Apply mandatory tags
+        Tags.of(alb).add("ApplicationName", app_name.lower().strip())
+        Tags.of(alb).add("Environment", environment)
+
         return alb
 
     def create_security_group(
-        self, scope, id, security_group_name, allow_all_outbound=True
+        self, scope, id, app_name, environment, security_group_name, allow_all_outbound=True
     ):
-        """
-        Create a fully compliant Security Group
+        """Create a Security Group resource
+        
+        Args:
+            id (str): the logical id of the newly created resource
+            
+            security_group_name (str): The security group name
+
+            allow_all_outbound (str): if the security group should enable outgoing traffic. Default=True
+    
+        Returns:
+            aws_ec2.SecurityGroup: the newly created security group
         """ 
         sg = _ec2.SecurityGroup(
             scope,
@@ -256,12 +328,22 @@ class Microservice(core.Construct):
             security_group_name=security_group_name,
             allow_all_outbound=allow_all_outbound,
         )
+
+        # Apply mandatory tags
+        Tags.of(sg).add("ApplicationName", app_name.lower().strip())
+        Tags.of(sg).add("Environment", environment)
+
         return sg
 
     # to update the userData definition after the first initialization
     def set_user_data_additional_variables(self, variables: dict):
-        """
-        Add the provided variables as environment variables for the user-data script
+        """Add the provided variables as environment variables for the user-data script
+        
+        Args:
+            variables (dict): the dict containing the variables to add to the user data as environment variables
+    
+        Returns:
+            aws_ec2.UserData: the updated user data
         """ 
         ADDITIONAL_VARIABLES_PLACEHOLDER = "#ADDITIONAL_VARIABLES_HERE"
         for vkey in variables:
@@ -281,8 +363,13 @@ class Microservice(core.Construct):
 
     # to update the userData definition after the first initialization
     def update_user_data(self, user_data: str):
-        """
-        Replaces the auto generated user-data with the provided one
+        """Replace the auto generated user-data with the provided one
+
+        Args:
+            user_data (aws_ec2.UserData): The User Data to use in the EC2 instance
+        
+        Returns:
+            aws_ec2.UserData: the provided user data
         """ 
         self.node.find_child(self.asg_name()).node.find_child(
             "LaunchConfig"
@@ -290,36 +377,36 @@ class Microservice(core.Construct):
 
     @staticmethod
     def pulse_cidr_as_peer(environments_parameters):
-        """
-        Returns the Pulse secure network as Peer 
+        """Returns the Pulse secure network as Peer
+
+        Args:
+            environments_parameters (dict): The dictionary containing the references to CSI AWS environments. This will simplify the environment promotions and enable a parametric development of the infrastructures.
+ 
+        Returns:
+            aws_ec2.Peer: The Pulse secure network as Peer resource 
         """ 
         fao_networks = environments_parameters["networking"]
         return _ec2.Peer.ipv4(fao_networks["pulse"])
 
     @staticmethod
     def fao_hq_clients_as_peer(environments_parameters):
-        """
-        Returns the HQ client subnet as Peer
+        """Returns the HQ client subnet as Peer
+
+        Args:
+            environments_parameters (dict): The dictionary containing the references to CSI AWS environments. This will simplify the environment promotions and enable a parametric development of the infrastructures.
+ 
+        Returns:
+            aws_ec2.Peer: the HQ client subnet as Peer resource 
         """ 
         fao_networks = environments_parameters["networking"]
         return _ec2.Peer.ipv4(fao_networks["fao_hq_clients"])
 
     def asg_name(self):
-        """Prints what the animals name is and what sound it makes.
+        """Return the Auto Scaling Group name
 
-        If the argument `sound` isn't passed in, the default Animal
-        sound is used.
-
-        Parameters
-        ----------
-        sound : str, optional
-            The sound the animal makes (default is None)
-
-        Raises
-        ------
-        NotImplementedError
-            If no sound is set for the animal or passed in as a
-            parameter.
+        Returns:
+            str: the Auto Scaling Group name
+       
         """
         return "asg"
 
@@ -327,7 +414,10 @@ class Microservice(core.Construct):
         self,
         scope: core.Construct,
         id: str,
-        main_component_name=None,
+        app_name,
+        environment,
+        environments_parameters,
+        main_component_name,
         will_be_public=False,
         will_send_email=False,
         ec2_ami_id=None,
@@ -343,15 +433,11 @@ class Microservice(core.Construct):
         downstream_security_group=None,
         downstream_port=None,
         ssl_certificate_arn=None,
-        toggle_to_trigger_rolling_update=None,
-        will_be_ha=False,
+        will_be_ha=True,
         ebs_volume_size=None,
         ebs_snapshot_id=None,
         will_use_efs=False,
-        environment=None,
-        environments_parameters=None,
         access_log_bucket_name="fao-elb-logs",
-        app_name=None,
         authorization_endpoint=None,
         token_endpoint=None,
         issuer=None,
@@ -361,12 +447,12 @@ class Microservice(core.Construct):
         additional_variables: dict = None,
         **kwargs
     ):
-        """
-        
-        """
-
         super().__init__(scope, id, **kwargs)
 
+        # Apply mandatory tags
+        Tags.of(self).add("ApplicationName", app_name, apply_to_launched_instances=True,)
+        Tags.of(self).add("Environment", environment, apply_to_launched_instances=True,)
+
         self.id = id
 
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Create conditions
@@ -495,9 +581,10 @@ class Microservice(core.Construct):
         self._fao_hq_clients_as_peer = self.fao_hq_clients_as_peer(
             self._environments_parameters
         )
-        upstream_security_group_as_peer = _ec2.SecurityGroup.from_security_group_id(
-            self, "upstream_security_group", upstream_security_group, mutable=True
-        )
+        if has_upstream:
+            upstream_security_group_as_peer = _ec2.SecurityGroup.from_security_group_id(
+                self, "upstream_security_group", upstream_security_group, mutable=True
+            )
 
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Create AWS resources
 
@@ -519,14 +606,14 @@ class Microservice(core.Construct):
         # Target group
         # ~~~~~~~~~~~~~~~~
         self._tg = self.create_target_group(
-            main_component_name, app_name, ec2_traffic_port, ec2_health_check_path
+            main_component_name, app_name, environment, ec2_traffic_port, ec2_health_check_path
         )
 
         # ~~~~~~~~~~~~~~~~
         # Elastic Load Balancer Security Group
         # ~~~~~~~~~~~~~~~~
         self.alb_security_group = self.create_security_group(
-            self, "alb_sg", app_name + '_' + main_component_name + "_alb_sg"
+            self, "alb_sg", app_name, environment, app_name + '_' + main_component_name + "_alb_sg"
         )
 
         if is_private:
@@ -577,7 +664,7 @@ class Microservice(core.Construct):
         # Elastic Load Balancer v2
         # ~~~~~~~~~~~~~~~~
         self.alb = self.create_load_balancer(
-            self, "alb", self.alb_security_group, internet_facing=is_public, load_balancer_name='-'.join([app_name, main_component_name, 'alb'])
+            self, "alb", app_name, environment, self.alb_security_group, internet_facing=is_public, load_balancer_name='-'.join([app_name, main_component_name, 'alb'])
         )
 
         # Listeners
@@ -617,6 +704,7 @@ class Microservice(core.Construct):
                     protocol=_alb.ApplicationProtocol.HTTPS,
                     certificates=[certificate],
                     default_target_groups=[self._tg],
+                    ssl_policy=_alb.SslPolicy.FORWARD_SECRECY_TLS12_RES
                 )
 
             if has_oidc:
@@ -643,6 +731,7 @@ class Microservice(core.Construct):
                     protocol=_alb.ApplicationProtocol.HTTPS,
                     certificates=[certificate],
                     default_action=oidc_listener_action,
+                    ssl_policy=_alb.SslPolicy.FORWARD_SECRECY_TLS12_RES
                 )
 
         # ~~~~~~~~~~~~~~~~
@@ -853,12 +942,14 @@ class Microservice(core.Construct):
             "_ApplicationName_", self._app_name
         )
         self.base_user_data = self.base_user_data.replace("_Environment_", environment)
-        self.base_user_data = self.base_user_data.replace(
-            "_S3CodeBucket_", s3_code_bucket_name
-        )
-        self.base_user_data = self.base_user_data.replace(
-            "_S3AssetsBucket_", s3_assets_bucket_name
-        )
+        if s3_code_bucket_name:
+            self.base_user_data = self.base_user_data.replace(
+                "_S3CodeBucket_", s3_code_bucket_name
+            )
+        if s3_assets_bucket_name: 
+            self.base_user_data = self.base_user_data.replace(
+                "_S3AssetsBucket_", s3_assets_bucket_name
+            )
         self.base_user_data = self.base_user_data.replace(
             "_IsAppPublic_", str(is_public)
         )
@@ -866,8 +957,16 @@ class Microservice(core.Construct):
         self.base_user_data = self.base_user_data.replace(
             "_UserDataS3Key_", user_data_s3_key
         )
+        # This is a Cloud Formation parameter. Change this value to automatically trigger a rolling update
+        toggle_to_trigger_rolling_update = core.CfnParameter(
+            self, 
+            "toggle_to_trigger_rolling_update_cfn_param", 
+            type="String",
+            description="Just alter with whatever value you want this parameter to trigger a rolling update",
+            default="__default__")
+        toggle_to_trigger_rolling_update.override_logical_id("ToggleToTriggerRollingUpdate"+main_component_name.capitalize())
         self.base_user_data = self.base_user_data.replace(
-            "_ToggleToTriggerRollingUpdate_", toggle_to_trigger_rolling_update
+            "_ToggleToTriggerRollingUpdate_", toggle_to_trigger_rolling_update.value_as_string
         )
         self.base_user_data = self.base_user_data.replace(
             "_EC2_TRAFFIC_PORT_", ec2_traffic_port
@@ -885,6 +984,10 @@ class Microservice(core.Construct):
         # ~~~~~~~~~~~~~~~~
         # Auto Scaling Group
         # ~~~~~~~~~~~~~~~~
+        if not ec2_ami_id:
+            ec2_ami_id = _ssm.StringParameter.value_for_string_parameter(
+            self, "/common/linuxHardenedAmi/latest")
+
         instance_type = _ec2.InstanceType(ec2_instance_type)
         machine_image = _ec2.GenericLinuxImage({"eu-west-1": ec2_ami_id})
         vpc_subnets = _ec2.SubnetSelection(availability_zones=[az_in_use])
@@ -933,6 +1036,7 @@ class Microservice(core.Construct):
                 wait_on_resource_signals=True,
             ),
         )
+        
 
         ags_config = self._asg.node.find_child("LaunchConfig")
         ags_config.add_property_override(
@@ -1077,4 +1181,4 @@ class Microservice(core.Construct):
                 peer=self.ec2_security_group,
                 connection=tcp_connection_downstream_port,
                 description="EC2 to downstream",
-            )
+            )
\ No newline at end of file
